\name{pcpattL}
\alias{pcpattL}
\title{
    Likert to Paired Comparison Patterns - Design Matrix
}
\description{
The function \code{pcpattL} converts a set of likert type responses measured on a commmon scale into
paired comparison patterns, returning a new data frame containing the design matrix for a loglinear
paired comparison model. Additionally, the
frequencies of these patterns are computed and are stored in the first
column of the data frame.  Optionally,
the function provides all necessary structures (commands, data/design
files) to fit the loglinear paired comparisons pattern model in GLIM,
which is often more efficient at fitting large loglinear models of this type.
}
\usage{
pcpattL(ctrl, dfr = NULL)
}
\arguments{
  \item{ctrl}{a list of control parameters  If
        \code{ctrl} is not correctly specified an error message is printed.}
  \item{dfr}{a dataframe conforming to the same specifications as the input
        data file (see below). The default is \code{NULL}, i.e., no dataframe
        is supplied in the call of \code{pcpattL} with the data instead supplied
        through the \code{datafile} element in the \code{ctrl} list.}
}
\details{

   Prior to the call of \code{pcpattL} the user has to provide a control list
   \code{ctrl} (see \bold{The Control List}) and data (either in the form of a
   dataframe or an external file) which has to conform to a certain structure
   (see \bold{Input Data}).\cr

   The typical usage is
     \preformatted{desmat <- pcpattL(ctrl)}
   or just \code{pcpattL(ctrl)} if only the GLIM output is wanted.

   The function \code{pcpattL} allows for different scenarios mainly concerning
   \itemize{
       \item \strong{responses.} The responses to the Likert type items are transformed
             to paired comparison responses by calculating the difference
             between each pair of the Likert items. These differences are then reduced to
             three response categories
             (\emph{preferred} -- \emph{undecided} -- \emph{not preferred}). The set of paired
             comparison responses represents a response pattern.
       \item \strong{item covariates.} The design matrix for the basic model has columns
             for the items (objects) and for undecided comparisons (i.e.,
             there is one dummy variable for each comparison to reflect if the
             corresponding item reponses are equal and 0 otherwise).
             Additionally, covariates for two way interaction between comparisons
             (i.e., for effects resulting from the dependency between two
             comparisons that have one item in common) can be obtained by setting
             \code{blnIntcovs = TRUE}.
       \item \strong{subject covariates.} For modelling different preference
             scales for the items according to characteristics of the respondents
             categorical subject covariates can be included in the design.
             The corresponding variables are defined as numerical vectors where
             the levels are specified with consecutive integers starting with 1.
             This format must be used in the input data file and is also used in
             all outputs.
       \item \strong{GLIM output}. If the user specifies \code{blnGLIMcmds = TRUE}
             two files are generated one of which contains all GLIM commands to fit
             a basic loglinear paired comparisons pattern model. The other contains
             the design matrix optionally including subject covariates. If
             dependency covariates are requested they are written to a third file.
             (Please note that the corresponding part of design matrix is transposed
             in the interactions output file to allow GLIM for using the \code{$array}
             facility in case of a large number of parameters to be estimated)
   }
}
\value{
  The output is a dataframe. Each row represents a unique response pattern.
  If subject covariates are specified, each row instead represents a particular
  combination of a unique covariate combination with a response pattern. All
  possible combinations are generated.\cr
  The first column contains the counts for the paired
  comparison response patterns and is labelled with \code{Y}. The next columns
  are the covariates for the items and the undecided category effects (one for
  each comparison). These are labelled \code{Uab}, where  \code{ab} denotes the
  comparison between items \code{a} and \code{b}. Optionally,
  covariates for dependencies between comparisons follow. The columns are labelled
  \code{Ia.bc} denoting the interaction of the comparisons between items \code{(a,b)}
  and \code{(a,c)} where the common item is \code{a}. If subject covariates are
  present they are in the rightmost columns and defined to be factors.\cr
%%  The response patterns themselves are not given in the output dataframe.
%%  However,
%%  the rows of the dataframe are in standard order, with the patterns varying within
%%  covariate combinations, and the patterns varying with the leftmost comparison
%%  (1,2), (1,3), (2,3), (1,4) etc varying fastest.
  Alternatively, the function \code{pcpattL} does not produce output in \R if GLIM output
  is requested via \code{blnGLIMcmds = TRUE}. The output is then written to
  the corresponding files (see \bold{The Control List} below).}
\references{Dittrich, R,, Francis, B.J., Hatzinger R., Katzenbeisser, W. (2007),
A Paired Comparison Approach for the Analysis of Sets of Likert Scale Responses. Statistical
Modelling, 7(1):3-28. }
\author{Reinhold Hatzinger}
\section{The Control List}{
   The argument \code{ctrl} is a list with elements described below. It must be defined prior
   to the call of \code{pcpattL}.

   \describe{
     \item{\code{datafile}}{If the data is to be read from a datafile,
            the path/name of the datafile to be read
          }
     \item{\code{nitems}}{the number of Likert items in the datafile
          }
     \item{\code{blnRevert}}{ If the Likert type responses are such that
          low values correspond to high agreement and high values
          to low agreement (e.g., (1) \emph{I strongly agree} \ldots
          (5)  \emph{I strongly disagree}) then \code{blnRevert}
          should be specified to be \code{FALSE}. Otherwise
          set \code{blnRevert = TRUE}.
          }
     \item{\code{blnIntcovs}}{generates covariates for interactions between comparisons if
          \code{blnIntcovs = TRUE}.
          }
     \item{\code{cov.sel}}{a character vector with the names of the subject covariates
          in the data file to be included into the design matrix.
          (example: \code{cov.sel = c("SEX", "AGE")}). If all covariates are to be
          included the specification can be abbreviated to \code{cov.sel = "ALL"}.
          For no covariates specify \code{cov.sel = ""}.
          }
     \item{\code{blnGLIMcmds}}{\code{TRUE}, if GLIM output is wanted. If \code{blnGLIMcmds = FALSE}
          the following items can be set to any value (such as a null text string) and are ignored.
          Please note that if \code{blnGLIMcmds}} is set to be \code{TRUE} there is no output in \R
          but instead goes to the the following files.
     \item{\code{glimCmdFile}}{name of the output file which will contain all necessary commands to
          fit a basic model (defining all structures and reading the
          necessary data).
          }
     \item{\code{outFile}}{name of the data/design file to be read into GLIM. It consists
           of the response frequencies and the covariates for the objects,
           the undecided comparison responses and the subject effects.
          }
     \item{\code{intFile}}{name of the design file for the interaction effects.
          This file is only generated if \code{blnIntcovs = TRUE}.
          }
   }
}
\section{Input Data}{
Input data is specified either through an external file (as specified through
\code{datafile} in \code{ctrl}) or through a dataframe via the argument \code{dfr}.
The input data file if specified must be a plain
text file with variable names in the first row as readable via the command
\code{read.table(datafile, header = TRUE)}. The leftmost columns must be the
responses to the Likert items optionally followed  by columns for
categorical
subject covariates. These have to be specified such that the categories are represented
by consecutive integers starting with 1. Missing values are treated such that
rows with one ore more NAs are removed from the data and a message is printed.
For an example see \code{\link{xmpl}} or the file \code{xmpl.dat} in the package's
\code{data/} directory.
}
%%
%%\note{
%%An alternative way to use \code{pcpattL} is
%%\preformatted{desmat <- pcpattL(ctrl, dfr = datframe)}
%%where \code{datframe} is a dataframe with exactly the same format as described for
%%the input data file.}
%%
\section{Warning}{Care has to be taken in case of a larger number of Likert items (\eqn{\ge 7})
and/or of covariate level combinations. The tractable size of the design matrix depends on the
working memory available to \R.}
\examples{
\dontrun{

## EXAMPLE 1: TYPICAL USAGE
## not run because input data file does not exist


# defining the ctrl list
# would be typically read from file using source()

testex1<-list(
   datafile     = "test/test.dat",
   nitems       = 5,
   blnRevert    = FALSE,
   blnIntcovs   = FALSE,

   cov.sel      = c("SEX","URB"),

   blnGLIMcmds  = TRUE,
   glimCmdFile  = "test/test.gli",
   outFile      = "test/test.design",
   intFile      = ""        # since blnIntcovs = FALSE
)

# call

pcpattL(testex1)

}

## EXAMPLE 2: WITH LOADED DATAFRAME

data(xmpl)   # example data in package

testex2<-list(
   datafile     = "",       # dataframe used
   nitems       = 3,
   blnRevert    = FALSE,
   blnIntcovs   = TRUE,

   cov.sel      = "ALL",

   blnGLIMcmds  = FALSE,    # no GLIM output
   glimCmdFile  = "",
   outFile      = "",
   intFile      = ""
)

dsgnmat <- pcpattL(testex2, xmpl)

print(head(dsgnmat))


## EXAMPLE 3: ILLUSTRATING THE ISSP2000 EXAMPLE
##   simplified version of the analysis as
##   given in Dittrich et.al.(2007)

data(issp2000)

testex3<-list(
   datafile     = "",
   nitems       = 6,
   blnRevert    = FALSE,
   blnIntcovs   = FALSE,

   cov.sel      = c("SEX","EDU"),

   blnGLIMcmds  = FALSE,
   glimCmdFile  = "",
   outFile      = "",
   intFile      = ""
)

design <- pcpattL(testex3, issp2000)


#  - fit null multinomial model (basic model for items without
#    subject covariates) through Poisson distribution.
#  - SEX:EDU parameters are nuisance parameters
#  - the last item (GENE) becomes a reference item
#    in the model and is aliased; all other items
#    are compared to this last item

# item parameters with undecided effects and no covariate effects.

summary(glm(y~SEX:EDU + CAR+IND+FARM+WATER+TEMP+GENE
                + U12+U13+U23+U14+U24+U34+U15+U25+U35+U45+U16+U26+U36+U46+U56,
             family=poisson, data=design))

# now add main effect of SEX on items

summary(glm(y~SEX:EDU + CAR+IND+FARM+WATER+TEMP+GENE
                + (CAR+IND+FARM+WATER+TEMP+GENE):SEX
                + U12+U13+U23+U14+U24+U34+U15+U25+U35+U45+U16+U26+U36+U46+U56,
             family=poisson, data=design))

\dontshow{
   testex2$cov.sel<-""
   dsgn2<-pcpattL(testex2, xmpl)
   print(head(dsgn2))
}

}
\keyword{misc}
